==> gitlab-tools-v0.2.0.tar.gz: gitlab-tools/bin/glx-clone.sh <==
#!/bin/bash
#
# gitlab-tools:bin/glx-clone.sh
#
set -u
set -o pipefail

PROJECT_HOME="$( dirname "$( dirname "$( realpath "${BASH_SOURCE[0]}" )" )" )"
declare -r PROJECT_HOME

# shellcheck disable=SC1090
source "${PROJECT_HOME}/apis/bash-common-api/bash-common-api.sh" || exit 1
# shellcheck disable=SC1090
source "${PROJECT_HOME}/bin/git-api/git-api.sh" || exit 1
# shellcheck disable=SC1090
source "${PROJECT_HOME}/bin/glt-api/project.sh" || exit 1

function usage {
  cat <<EOF
Help to clone repositories for back propose or any automations

Usage:
  ${BASH_SOURCE[0]} action [options]

Action:
  -h, --help            Display help message and exit
  all,                  Clone all available repositories
  group ID              Clone all repositories with given group id.

Options:
  -g GITLAB, --gitlab GITLAB
                        Which configuration section should be used. If not
                        defined, the default selection will be used. (python-gitlab)
  --bare                Clone using --bare option (git)
  --ssh                 Use ssh to clone repositories
  --ssh-key KEYFILE     Define a custom ssh key for clone over ssh.
  --http                Use http(s) to clone repositories
  -d ROOTDIR, --root-dir ROOTDIR
                        Define destination root directory (if not set, use current directory)

EOF
}

function clone_projects {
  local -r urls="$1"
  local -r bare="$2"
  local -r root_output_directory="$3"
  local -r ssh_key="$4"
  local    group=
  local    project=
  local    url=

  mkdir -p "${root_output_directory}"
  pushd "${root_output_directory}" >/dev/null || exit 1

  while read -r url ; do
    if [ -n "${url}" ] ; then
      group="$( rev <<<"${url}" | cut -d '/' -f2 | cut -d ':' -f1 | rev )"
      project="$( rev <<<"${url}" | cut -d '.' -f2 | cut -d '/' -f1 | rev )"

      echoinfo "${group} / ${project} : ${url}"

      mkdir -p "${group}"
      pushd "${group}" >/dev/null || exit 1

      git_clone "${bare}" "${url}" "${ssh_key}"

      popd >/dev/null || exit 1
    fi
  done <<<"${urls}"

  popd >/dev/null || exit 1
}

function clone_all_projects {
  local -r proto="$1"
  local -r bare="$2"
  local -r root_output_directory="$3"
  local -r ssh_key="$4"

  clone_projects \
    "$(
        gitlab_get_all_projects "${gitlab}" |
        jq -r ".[].${proto}"
    )" \
    "${bare}" \
    "${root_output_directory}" \
    "${ssh_key}"
}

function clone_all_projects_for_groupid {
  local -r proto="$1"
  local -r bare="$2"
  local -r root_output_directory="$3"
  local -r ssh_key="$4"
  local -ri group_id="$5"

  clone_projects \
    "$(
        gitlab_get_projects_by_groupid "${gitlab}" "${group_id}" |
        jq -r ".[].${proto}"
    )" \
    "${bare}" \
    "${root_output_directory}" \
    "${ssh_key}"
}

function main {
  local action=
  local bare=
  local gitlab=
  local proto=
  local root_output_directory=
  local ssh_key=
  local -i group_id=0
  root_output_directory="$( pwd )"

  while [ $# -gt 0 ] ; do
    local param="$1"
    shift

    case "${param}" in
      '-h'|'--help')
        usage
        exit 0
        ;;

      'all'     ) action='all' ;;
      'group'   ) action='group' ; group_id="$( P "$@" )" && shift ;;

      '--gitlab'        ) gitlab="$( P "$@" )" && shift ;;
      '--bare'          ) bare='--bare' ;;
      '--ssh'           ) proto='ssh_url_to_repo' ;;
      '--http'          ) proto='http_url_to_repo' ;;
      '--ssh-key'       ) ssh_key="$( P "$@" )" && shift ;;
      '-d'|'--root-dir' ) root_output_directory="$( P "$@" )" && shift ;;

      *)
        # unknown option
        echofatal "Unknown parameter '${param}'"
        usage >&2
        exit 1
        ;;
    esac
  done

  if [ -z "${proto}" ] ; then
    echofatal 'Missing parameter --ssh or --http'
    usage >&2
    exit 1
  fi
  if [ -n "${ssh_key}" ] &&  [ ! -f "${ssh_key}" ] ; then
    echofatal "File not found '${ssh_key}'"
    exit 1
  fi
  if [ "${proto}" != 'ssh_url_to_repo' ] ; then
    if [ -n "${ssh_key}" ] ; then
      echofatal "--ssh-key '${ssh_key}' is define, but not --ssh"
      exit 1
    fi
  fi

  case "${action}" in
    'all')
      clone_all_projects "${proto}" "${bare}" "${root_output_directory}" "${ssh_key}"
      ;;

    'group')
      clone_all_projects_for_groupid "${proto}" "${bare}" "${root_output_directory}" "${ssh_key}" "${group_id}"
      ;;

    *)
      echofatal 'Missing action.'
      usage >&2
      exit 1
      ;;
  esac
}

main "$@" || exit 1

==> gitlab-tools-v0.2.1.tar.gz: gitlab-tools/bin/glx-clone.sh <==
#!/bin/bash
#
# gitlab-tools:bin/glx-clone.sh
#
set -u
set -o pipefail

PROJECT_HOME="$( dirname "$( dirname "$( realpath "${BASH_SOURCE[0]}" )" )" )"
declare -r PROJECT_HOME

# shellcheck disable=SC1090
source "${PROJECT_HOME}/apis/bash-common-api/bash-common-api.sh" || exit 1
# shellcheck disable=SC1090
source "${PROJECT_HOME}/bin/git-api/git-api.sh" || exit 1
# shellcheck disable=SC1090
source "${PROJECT_HOME}/bin/glt-api/project.sh" || exit 1

function usage {
  cat <<EOF
Help to clone repositories for back propose or any automations

Usage:
  ${BASH_SOURCE[0]} action [options]

Action:
  -h, --help            Display help message and exit
  all,                  Clone all available repositories
  group ID              Clone all repositories with given group id.

Options:
  -g GITLAB, --gitlab GITLAB
                        Which configuration section should be used. If not
                        defined, the default selection will be used. (python-gitlab)
  --bare                Clone using --bare option (git)
  --ssh                 Use ssh to clone repositories
  --ssh-key KEYFILE     Define a custom ssh key for clone over ssh.
  --http                Use http(s) to clone repositories
  -d ROOTDIR, --root-dir ROOTDIR
                        Define destination root directory (if not set, use current directory)

EOF
}

function clone_projects {
  local -r urls="$1"
  local -r bare="$2"
  local -r root_output_directory="$3"
  local -r ssh_key="$4"
  local    group=
  local    project=
  local    url=

  mkdir -p "${root_output_directory}"
  pushd "${root_output_directory}" >/dev/null || exit 1

  while read -r url ; do
    if [ -n "${url}" ] ; then
      group="$( rev <<<"${url}" | cut -d '/' -f2 | cut -d ':' -f1 | rev )"
      project="$( rev <<<"${url}" | cut -d '.' -f2 | cut -d '/' -f1 | rev )"

      echoinfo "${group} / ${project} : ${url}"

      mkdir -p "${group}"
      pushd "${group}" >/dev/null || exit 1

      git_clone "${bare}" "${url}" "${ssh_key}"

      popd >/dev/null || exit 1
    fi
  done <<<"${urls}"

  popd >/dev/null || exit 1
}

function clone_all_projects {
  local -r proto="$1"
  local -r bare="$2"
  local -r root_output_directory="$3"
  local -r ssh_key="$4"

  clone_projects \
    "$(
        gitlab_get_all_projects "${gitlab}" |
        jq -r ".[].${proto}"
    )" \
    "${bare}" \
    "${root_output_directory}" \
    "${ssh_key}"
}

function clone_all_projects_for_groupid {
  local -r proto="$1"
  local -r bare="$2"
  local -r root_output_directory="$3"
  local -r ssh_key="$4"
  local -ri group_id="$5"

  clone_projects \
    "$(
        gitlab_get_projects_by_groupid "${gitlab}" "${group_id}" |
        jq -r ".[].${proto}"
    )" \
    "${bare}" \
    "${root_output_directory}" \
    "${ssh_key}"
}

function main {
  local    action=
  local    bare=
  local    gitlab=
  local    param=
  local    proto=
  local    root_output_directory=
  local    ssh_key=
  local -i group_id=0

  root_output_directory="$( pwd )"

  while [ "$#" -gt 0 ] ; do
    param="$1"
    shift

    case "${param}" in
      '-h'|'--help')
        usage
        exit 0
        ;;

      'all'     ) action='all' ;;
      'group'   ) action='group' ; group_id="$( P "$@" )" && shift ;;

      '--gitlab'        ) gitlab="$( P "$@" )" && shift ;;
      '--bare'          ) bare='--bare' ;;
      '--ssh'           ) proto='ssh_url_to_repo' ;;
      '--http'          ) proto='http_url_to_repo' ;;
      '--ssh-key'       ) ssh_key="$( P "$@" )" && shift ;;
      '-d'|'--root-dir' ) root_output_directory="$( P "$@" )" && shift ;;

      *)
        echofatal "Unknown option: '${param}'"
        usage >&2
        exit 1
        ;;
    esac
  done

  if [ -z "${proto}" ] ; then
    echofatal 'Missing parameter --ssh or --http'
    usage >&2
    exit 1
  fi
  if [ -n "${ssh_key}" ] &&  [ ! -f "${ssh_key}" ] ; then
    echofatal "File not found '${ssh_key}'"
    exit 1
  fi
  if [ "${proto}" != 'ssh_url_to_repo' ] ; then
    if [ -n "${ssh_key}" ] ; then
      echofatal "--ssh-key '${ssh_key}' is define, but not --ssh"
      exit 1
    fi
  fi

  case "${action}" in
    'all')
      clone_all_projects "${proto}" "${bare}" "${root_output_directory}" "${ssh_key}"
      ;;

    'group')
      clone_all_projects_for_groupid "${proto}" "${bare}" "${root_output_directory}" "${ssh_key}" "${group_id}"
      ;;

    *)
      echofatal 'Missing action.'
      usage >&2
      exit 1
      ;;
  esac
}

main "$@" || exit 1
